
"""
Middleware tests
"""
import unittest
import amanwa32
import webtest

from webob import Response
# from google.appengine.ext import testbed, ndb

import settings
from core.handlers import BaseHandler
from core.middleware import Middleware


class TestRequestMiddleware(Middleware):
    def process_request(self, request):
        request.value = 'request'


class TestResponseMiddleware(Middleware):
    def process_response(self, request, response):
        response.body = 'response'


class APIUserHandler(BaseHandler):
    def get(self, *args, **kwargs):
        return Response(body=self.request.value)

    def post(self, *args, **kwargs):
        return Response()

'''
class APIHandlerTestCase(unittest.TestCase):
    def setUp(self):
        self.testbed = testbed.Testbed()
        self.testbed.activate()
        self.testbed.init_datastore_v3_stub()
        self.testbed.init_memcache_stub()
        self.testbed.init_app_identity_stub()

        ndb.get_context().clear_cache()

        app = amanwa32.WSGIApplication([
            ('/user/', APIUserHandler)
        ])
        self.testapp = webtest.TestApp(app)

    def test_request_middleware(self):
        setattr(settings, 'MIDDLEWARE_CLASSES', ['core.middleware_test.TestRequestMiddleware'])

        response = self.testapp.get('/user/')
        self.assertEquals(response.body, 'request')

    def test_response_middleware(self):
        setattr(settings, 'MIDDLEWARE_CLASSES', ['core.middleware_test.TestResponseMiddleware'])

        response = self.testapp.post('/user/')
        self.assertEquals(response.body, 'response')
'''

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright 2011 amanwa32 AUTHORS.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
amanwa32_extras.routes
=====================

Extra route classes for amanwa32.
"""
import six
from six.moves.urllib import parse

import amanwa32
from webob import exc


class MultiRoute(object):
    """Base class for routes with nested routes."""

    routes = None
    children = None
    match_children = None
    build_children = None

    def __init__(self, routes):
        self.routes = routes

    def get_children(self):
        if self.children is None:
            self.children = []
            for route in self.routes:
                for r in route.get_routes():
                    self.children.append(r)

        for rv in self.children:
            yield rv

    def get_match_children(self):
        if self.match_children is None:
            self.match_children = []
            for route in self.get_children():
                for r in route.get_match_routes():
                    self.match_children.append(r)

        for rv in self.match_children:
            yield rv

    def get_build_children(self):
        if self.build_children is None:
            self.build_children = {}
            for route in self.get_children():
                for n, r in route.get_build_routes():
                    self.build_children[n] = r

        for rv in six.iteritems(self.build_children):
            yield rv

    get_routes = get_children
    get_match_routes = get_match_children
    get_build_routes = get_build_children


class DomainRoute(MultiRoute):
    """A route used to restrict route matches to a given domain or subdomain.

    For example, to restrict routes to a subdomain of the appspot domain::

        app = WSGIApplication([
            DomainRoute('<subdomain>.app-id.appspot.com', [
                Route('/foo', 'FooHandler', 'subdomain-thing'),
            ]),
            Route('/bar', 'BarHandler', 'normal-thing'),
        ])

    The template follows the same syntax used by :class:`amanwa32.Route` and
    must define named groups if any value must be added to the match results.
    In the example above, an extra `subdomain` keyword is passed to the
    handler, but if the regex didn't define any named groups, nothing would
    be added.
    """

    def __init__(self, template, routes):
        """Initializes a URL route.

        :param template:
            A route template to match against ``environ['SERVER_NAME']``.
            See a syntax description in :meth:`amanwa32.Route.__init__`.
        :param routes:
            A list of :class:`amanwa32.Route` instances.
        """
        super(DomainRoute, self).__init__(routes)
        self.template = template

    def get_match_routes(self):
        # This route will do pre-matching before matching the nested routes!
        yield self

    def match(self, request):
        # Use SERVER_NAME to ignore port number that comes with request.host?
        # host_match = self.regex.match(request.host.split(':', 1)[0])
        host_match = self.regex.match(request.environ['SERVER_NAME'])

        if host_match:
            args, kwargs = amanwa32._get_route_variables(host_match)
            return _match_routes(self.get_match_children, request, None,
                                 kwargs)

    @amanwa32.cached_property
    def regex(self):
        regex, reverse_template, args_count, kwargs_count, variables = \
            amanwa32._parse_route_template(self.template,
                                          default_sufix='[^\.]+')
        return regex


class NamePrefixRoute(MultiRoute):
    """The idea of this route is to set a base name for other routes::

        app = WSGIApplication([
            NamePrefixRoute('user-', [
                Route('/users/<user:\w+>/', UserOverviewHandler, 'overview'),
                Route('/users/<user:\w+>/profile', UserProfileHandler,
                      'profile'),
                Route('/users/<user:\w+>/projects', UserProjectsHandler,
                      'projects'),
            ]),
        ])

    The example above is the same as setting the following routes, just more
    convenient as you can reuse the name prefix::

        app = WSGIApplication([
            Route('/users/<user:\w+>/', UserOverviewHandler, 'user-overview'),
            Route('/users/<user:\w+>/profile', UserProfileHandler,
                  'user-profile'),
            Route('/users/<user:\w+>/projects', UserProjectsHandler,
                  'user-projects'),
        ])
    """

    _attr = 'name'

    def __init__(self, prefix, routes):
        """Initializes a URL route.

        :param prefix:
            The prefix to be prepended.
        :param routes:
            A list of :class:`amanwa32.Route` instances.
        """
        super(NamePrefixRoute, self).__init__(routes)
        self.prefix = prefix
        # Prepend a prefix to a route attribute.
        for route in self.get_routes():
            setattr(route, self._attr, prefix + getattr(route, self._attr))


class HandlerPrefixRoute(NamePrefixRoute):
    """Same as :class:`NamePrefixRoute`, but prefixes the route handler."""

    _attr = 'handler'


class PathPrefixRoute(NamePrefixRoute):
    """Same as :class:`NamePrefixRoute`, but prefixes the route path.

    For example, imagine we have these routes::

        app = WSGIApplication([
            Route('/users/<user:\w+>/', UserOverviewHandler,
                  'user-overview'),
            Route('/users/<user:\w+>/profile', UserProfileHandler,
                  'user-profile'),
            Route('/users/<user:\w+>/projects', UserProjectsHandler,
                  'user-projects'),
        ])

    We could refactor them to reuse the common path prefix::

        app = WSGIApplication([
            PathPrefixRoute('/users/<user:\w+>', [
                Route('/', UserOverviewHandler, 'user-overview'),
                Route('/profile', UserProfileHandler, 'user-profile'),
                Route('/projects', UserProjectsHandler, 'user-projects'),
            ]),
        ])

    This is not only convenient, but also performs better: the nested routes
    will only be tested if the path prefix matches.
    """

    _attr = 'template'

    def __init__(self, prefix, routes):
        """Initializes a URL route.

        :param prefix:
            The prefix to be prepended. It must start with a slash but not
            end with a slash.
        :param routes:
            A list of :class:`amanwa32.Route` instances.
        """
        assert prefix.startswith('/') and not prefix.endswith('/'), \
            'Path prefixes must start with a slash but not end with a slash.'
        super(PathPrefixRoute, self).__init__(prefix, routes)

    def get_match_routes(self):
        # This route will do pre-matching before matching the nested routes!
        yield self

    def match(self, request):
        if not self.regex.match(parse.unquote(request.path)):
            return None

        return _match_routes(self.get_match_children, request)

    @amanwa32.cached_property
    def regex(self):
        regex, reverse_template, args_count, kwargs_count, variables = \
            amanwa32._parse_route_template(self.prefix + '<:/.*>')
        return regex


class RedirectRoute(amanwa32.Route):
    """A convenience route class for easy redirects.

    It adds redirect_to, redirect_to_name and strict_slash options to
    :class:`amanwa32.Route`.
    """

    def __init__(self, template, handler=None, name=None, defaults=None,
                 build_only=False, handler_method=None, methods=None,
                 schemes=None, redirect_to=None, redirect_to_name=None,
                 strict_slash=False):
        """Initializes a URL route. Extra arguments compared to
        :meth:`amanwa32.Route.__init__`:

        :param redirect_to:
            A URL string or a callable that returns a URL. If set, this route
            is used to redirect to it. The callable is called passing
            ``(handler, *args, **kwargs)`` as arguments. This is a
            convenience to use :class:`RedirectHandler`. These two are
            equivalent::

                route = Route('/foo', handler=amanwa32.RedirectHandler,
                              defaults={'_uri': '/bar'})
                route = Route('/foo', redirect_to='/bar')

        :param redirect_to_name:
            Same as `redirect_to`, but the value is the name of a route to
            redirect to. In the example below, accessing '/hello-again' will
            redirect to the route named 'hello'::

                route = Route('/hello', handler=HelloHandler, name='hello')
                route = Route('/hello-again', redirect_to_name='hello')

        :param strict_slash:
            If True, redirects access to the same URL with different trailing
            slash to the strict path defined in the route. For example, take
            these routes::

                route = Route('/foo', FooHandler, strict_slash=True)
                route = Route('/bar/', BarHandler, strict_slash=True)

            Because **strict_slash** is True, this is what will happen:

            - Access to ``/foo`` will execute ``FooHandler`` normally.
            - Access to ``/bar/`` will execute ``BarHandler`` normally.
            - Access to ``/foo/`` will redirect to ``/foo``.
            - Access to ``/bar`` will redirect to ``/bar/``.
        """
        super(RedirectRoute, self).__init__(
            template, handler=handler, name=name, defaults=defaults,
            build_only=build_only, handler_method=handler_method,
            methods=methods, schemes=schemes)

        if strict_slash and not name:
            raise ValueError('Routes with strict_slash must have a name.')

        self.strict_slash = strict_slash
        self.redirect_to_name = redirect_to_name

        if redirect_to is not None:
            assert redirect_to_name is None
            self.handler = amanwa32.RedirectHandler
            self.defaults['_uri'] = redirect_to

    def get_match_routes(self):
        """Generator to get all routes that can be matched from a route.

        :yields:
            This route or all nested routes that can be matched.
        """
        if self.redirect_to_name:
            main_route = self._get_redirect_route(name=self.redirect_to_name)
        else:
            main_route = self

        if not self.build_only:
            if self.strict_slash is True:
                if self.template.endswith('/'):
                    template = self.template[:-1]
                else:
                    template = self.template + '/'

                yield main_route
                yield self._get_redirect_route(template=template)
            else:
                yield main_route

    def _get_redirect_route(self, template=None, name=None):
        template = template or self.template
        name = name or self.name
        defaults = self.defaults.copy()
        defaults.update({
            '_uri': self._redirect,
            '_name': name,
        })
        new_route = amanwa32.Route(template, amanwa32.RedirectHandler,
                                  defaults=defaults)
        return new_route

    def _redirect(self, handler, *args, **kwargs):
        # Get from request because args is empty if named routes are set?
        # args, kwargs = (handler.request.route_args,
        #                 handler.request.route_kwargs)
        kwargs.pop('_uri', None)
        kwargs.pop('_code', None)
        return handler.uri_for(kwargs.pop('_name'), *args, **kwargs)


def _match_routes(iter_func, request, extra_args=None, extra_kwargs=None):
    """Tries to match a route given an iterator."""
    method_not_allowed = False
    for route in iter_func():
        try:
            match = route.match(request)
            if match:
                route, args, kwargs = match
                if extra_args:
                    args += extra_args

                if extra_kwargs:
                    kwargs.update(extra_kwargs)

                return route, args, kwargs
        except exc.HTTPMethodNotAllowed:
            method_not_allowed = True

    if method_not_allowed:
        raise exc.HTTPMethodNotAllowed()



"""
Handlers test
"""
import unittest
import amanwa32
import webtest

from webob import Response

from core.exceptions import BadRequest, NotFound
from core.decorators import action
from core.response import JsonResponse
from core.handlers import RestHandler, BaseHandler
from core.router import Router


class User(ndb.Model):
    email = ndb.StringProperty()
    last_name = ndb.StringProperty()
    first_name = ndb.StringProperty()
    age = ndb.IntegerProperty()


class UserApi(RestHandler):
    """
    User example API
    """
    def list(self, request):
        users = User.query().fetch()

        return JsonResponse(data={
            'results': [
                user.to_dict()
                for user in users
            ],
            'count': len(users)
        })

    def create(self, request):
        data = self.get_body()

        try:
            new_user = User(email=data['email'])
            new_user.put()

            return JsonResponse(
                status=201,
                data=new_user.to_dict()
            )
        except Exception as err:
            raise BadRequest

    def retrieve(self, request, key=None):
        user = User.query(User.email == key).fetch()

        if user:
            return JsonResponse(data=user[0].to_dict())

        raise NotFound

    def destroy(self, request, key=None):
        user_key = User.query(User.email == key).fetch(keys_only=True)

        if user_key:
            user_key[0].delete()
            return JsonResponse(status=204)

        raise NotFound

    @action(methods=['GET'], detail=False)
    def toto(self, *args, **kwargs):
        return Response(body='toto')

    @action(methods=['GET'], detail=True)
    def tata(self, request, key):
        return Response(body=key)


class DynamicAPIModelHandlerTestCase(unittest.TestCase):
    def setUp(self):
        self.testbed = testbed.Testbed()
        self.testbed.activate()
        self.testbed.init_datastore_v3_stub()
        self.testbed.init_memcache_stub()

        ndb.get_context().clear_cache()

        router = Router()
        router.register('user', UserApi)
        app = amanwa32.WSGIApplication(router.urls)
        self.testapp = webtest.TestApp(app)
        self.entity = User(
            email='admin@test.com',
            first_name='admin',
            last_name='test',
            age=28
        )
        self.entity.put()

    def test_dispatch_toto(self):
        response = self.testapp.get('/user/toto')
        self.assertEquals(response.body, 'toto')

    def test_dispatch_toto_bad_method(self):
        response = self.testapp.put('/user/toto', expect_errors=True)
        self.assertEquals(response.status_code, 404)

    def test_dispatch_toto_slash(self):
        response = self.testapp.get('/user/toto/')
        self.assertEquals(response.body, 'toto')

    def test_dispatch_tata(self):
        response = self.testapp.get('/user/admin@test.com/tata')
        self.assertEquals(response.body, 'admin@test.com')

    def test_dispatch_tata_with_slash(self):
        response = self.testapp.get('/user/admin@test.com/tata/')
        self.assertEquals(response.body, 'admin@test.com')

    def test_dispatch_tata_bad_method(self):
        response = self.testapp.post('/user/admin@test.com/tata', expect_errors=True)
        self.assertEquals(response.status_code, 404)


class APIUserHandler(BaseHandler):
    def get(self, *args, **kwargs):
        return Response(body='get')

    def post(self, *args, **kwargs):
        return Response(body='post')

    def put(self, *args, **kwargs):
        return Response(body='put')

    def delete(self, *args, **kwargs):
        return Response(body='delete')

    def patch(self, *args, **kwargs):
        return Response(body='patch')


class APIHandlerTestCase(unittest.TestCase):
    def setUp(self):
        self.testbed = testbed.Testbed()
        self.testbed.activate()
        self.testbed.init_datastore_v3_stub()
        self.testbed.init_memcache_stub()

        ndb.get_context().clear_cache()

        router = Router()
        router.register('user', APIUserHandler)
        app = amanwa32.WSGIApplication(router.urls)
        self.testapp = webtest.TestApp(app)

    def test_dispatch_get(self):
        response = self.testapp.get('/user/')
        self.assertEquals(response.body, 'get')

    def test_dispatch_post(self):
        response = self.testapp.post('/user/')
        self.assertEquals(response.body, 'post')

    def test_dispatch_put(self):
        response = self.testapp.put('/user/')
        self.assertEquals(response.body, 'put')

    def test_dispatch_patch(self):
        response = self.testapp.patch('/user/')
        self.assertEquals(response.body, 'patch')

    def test_dispatch_delete(self):
        response = self.testapp.delete('/user/')
        self.assertEquals(response.body, 'delete')


if __name__ == '__main__':
    unittest.main()



"""
Core handlers for amanwa32 simple use
"""
import json
import logging
import traceback

import re
import amanwa32
from webob import Response

from core.exceptions import (
    NotFound,
    NotAuthorized,
    MissingParameter,
    Forbidden,
    BadRequest,
    MissingBody,
    DispatchError
)
from core.middleware import get_middlewares
from core.response import JsonResponse, CorsResponse
from helpers.decorators import cached_property
import settings


class BaseCronHandler(amanwa32.RequestHandler):
    """
    Base cron handler
    """
    def dispatch(self):
        """
        Change dispatch default behaviour and checks if cron header is present
        """
        if self.is_cron():
            return super(BaseCronHandler, self).dispatch()
        else:
            return Response(status=400, json_body={
                'error': 'This route is only callable with google cloud cron jobs'
            })

    def is_cron(self):
        """
        Verify cron headers, in local development always return TRUE
        """
        return self.request.headers.get('X-Appengine-Cron', 'false') == 'true' or settings.IS_LOCAL


class BaseHandler(amanwa32.RequestHandler):
    """
    BaseHandler with some intelligence & automation
    """
    def __init__(self, *args, **kwargs):
        super(BaseHandler, self).__init__(*args, **kwargs)
        self.route_args = {}
        self.env = {}
        self.query_params = {}
        self.user = None
        self.action = None

    @cached_property
    def middlewares(self):
        """
        Cached property that load all middlewares
        :return: :py:class:`list` of :py:class:`Middleware` instances
        """
        return get_middlewares()

    def dispatch(self):
        """
        Function called to make the dispatch job.
        Calls internally :py:meth:`core.handlers.BaseHandler.apply_dispatch`
        :return: :py:class:`webob.Response`
        """
        self.action = self.request.method.lower()

        return self.apply_dispatch()

    def apply_dispatch(self):
        """
        Does the hole mechanism of dispatch.
        Loops over all middlewares before and after the dispatch method call
        .. code-block:: python
            for middleware in self.middlewares:
                middleware.process_request(self.request)
            self.route_args = self.get_route_args()
            response = self.do_dispatch()
            for middleware in self.middlewares:
                middleware.process_response(self.request, response)
        :return: :py:class:`core.response.Response`
        """
        try:
            self.query_params = dict(self.request.GET)
            self.env = self.request.GET.env

            for middleware in self.middlewares:
                if middleware.activate_on_method(self.request.method.upper()):
                    middleware.process_request(self.request)

            self.route_args = self.get_route_args()
            response = self.do_dispatch()

            for middleware in self.middlewares:
                if middleware.activate_on_method(self.request.method.upper()):
                    middleware.process_response(self.request, response)

            return response
        except NotFound as nf:
            return JsonResponse(status=404, data=str(nf) or 'Not found')
        except NotAuthorized as na:
            return JsonResponse(status=403, data=str(na) or 'Not authorized')
        except Forbidden as fo:
            return JsonResponse(status=401, data=str(fo) or 'Forbidden')
        except MissingParameter as mp:
            return JsonResponse(status=400, data=str(mp) or 'Missing parameter')
        except BadRequest as br:
            return JsonResponse(status=400, data=str(br) or 'Bad request')
        except MissingBody as mb:
            return JsonResponse(status=400, data=str(mb) or 'Request is missing a body')
        except ValueError as ve:
            return JsonResponse(status=400, data=str(ve) or 'Value error')
        except DispatchError:
            return Response(status=404)
        except Exception as err:
            logging.error('%s: %s',
                          err.__class__.__name__,
                          str(err))
            logging.error(traceback.format_exc())
            return self.handle_exception(err, self.app.debug)

    def check_action_permissions(self):
        """
        Overridable method on each handler. Allow top level check of permissions.
        For access control, override this method with your logic and return False if
        access should not be given. It will internally raise an :py:class:`core.exceptionsNotAuthorized` and
        return a 401 response
        :return: :py:class:`bool`
        """
        return True

    def do_dispatch(self):
        """
        Function that does the dispatch job
        :return: :py:class:`webob.Response`
        """
        if self.check_action_permissions() is False:
            raise NotAuthorized()

        return super(BaseHandler, self).dispatch()

    def get_body(self):
        """
        Tries to parse the request body as JSON
        :return: JSON serialized data as :py:class:`dict` instance.
        """
        _cached_body = getattr(self, '__cached_body', None)

        if _cached_body is not None:
            return _cached_body

        if self.request.body:
            try:
                json_body = json.loads(self.request.body)
                setattr(self, '__cached_body', json_body)
                return json_body
            except Exception as err:
                logging.warning('Request is missing a body: %s -> %s',
                                err.__class__.__name__, str(err))
        raise MissingBody('Request is missing a body')

    def get_route_args(self):
        """
        Parses the route arguments based on the handler url
        :return: route arguments in a :py:class:`dict` instance.
        For instance if the view url is ``r'/resource/(?P<slug>[^/]+)/'`` and the calling url is ``GET /resource/my-slug/``
        The returned dict will be formatted like this
        .. code-block:: python
           {'slug': 'my-slug'}
        .. note::
            Those route arguments are then available in each of the Handler methods in the self.route_args attribute
        """
        return self.request.route.regex.search(self.request.upath_info).groupdict()

    @classmethod
    def get_extra_actions(cls):
        actions = []

        for _method in cls.__dict__.values():
            if getattr(_method, 'is_dynamic_action', False):
                actions.append(_method)
        return actions


class RestHandler(BaseHandler):
    def list(self, request):
        """
        List operation
        :return: :py:class:`core.response.JsonResponse`
        """
        raise NotImplementedError

    def create(self, request):
        """
        :return: :py:class:`core.response.JsonResponse`
        """
        raise NotImplementedError

    def retrieve(self, request, key=None):
        """
        :return: :py:class:`core.response.JsonResponse`
        """
        raise NotImplementedError

    def update(self, request, key=None):
        """
        :return: :py:class:`core.response.JsonResponse`
        """
        raise NotImplementedError

    def partial_update(self, request, key=None):
        """
        :return: :py:class:`core.response.JsonResponse`
        """
        raise NotImplementedError

    def destroy(self, request, key=None):
        """
        :return: :py:class:`core.response.JsonResponse`
        """
        raise NotImplementedError

    def do_dispatch(self):
        """
        Dispatches the request.
        This will first check if there's a handler_method defined in the
        matched route, and if not it'll use the method correspondent to the
        request method (``get()``, ``post()`` etc).
        """
        if self.request.method.upper() == 'OPTIONS':
            return CorsResponse()

        route_args = self.get_route_args()
        route_kwargs = {}
        matched_url = None
        for url in getattr(self, 'urls', []):
            if re.match(url.regex, self.request.path_info):
                matched_url = url
                break

        if matched_url is None:
            raise NotFound('URL not found on this server')

        if matched_url.route.detail is True:
            try:
                route_kwargs['key'] = route_args['key']
            except Exception:
                return JsonResponse(status=400, data='Given urlsafe is invalid')

        if self.request.method.lower() not in matched_url.route.mapping.keys():
            raise DispatchError('Invalid method {}'.format(self.request.method))

        self.action = matched_url.route.mapping[self.request.method.lower()]
        method = getattr(self, self.action, None)
        # The handler only receives *args if no named variables are set.
        args, kwargs = route_args, route_kwargs
        if kwargs:
            args = ()

        if self.check_action_permissions() is False:
            raise NotAuthorized()
        return method(self, *args, **kwargs)




from codecs import open

from setuptools import setup




REQUIREMENTS = [
    'webob>=1.2.0',
    'six>=1.10.0'
]

EXTRA_REQUIREMENTS = {
    'jinja2>=2.4',
    'Babel>=2.2',
    'pytz>=2015.7'
}

setup(
    name='amanwa32',
    version='1.0.5',
    license='Apache Software License',
    description="testing wa3 custom",
    long_description='LONG_DESCRIPTION',
    author='a',
    author_email='angrish_aman@yahoo.com',
    zip_safe=False,
    platforms='any',
    py_modules=[
        'amanwa32',
    ],
    packages=[
        'amanwa32',
        'amanwa32_extras',
        #'amanwa32_extras.appengine',
        #'amanwa32_extras.appengine.auth',
    ],
    include_package_data=True,
    install_requires=REQUIREMENTS,
    extras_require={'extras': EXTRA_REQUIREMENTS},
    classifiers=[
       
    ]
)



"""
Serializers test
"""
import unittest

from core.decorators import action
from core.response import JsonResponse
from core.router import Router
from core.handlers import RestHandler, BaseHandler


class TestHandler(BaseHandler):
    pass


class TestRestHandler(RestHandler):
    pass


class TestDynamicRestHandler(RestHandler):
    @action(methods=['GET'], detail=False)
    def toto(self, *args, **kwargs):
        return JsonResponse(data={'ok': 42})


class BaseTestCase(unittest.TestCase):
    def test_dynamic_route(self):
        router = Router()
        handler_name = 'handler'
        router.register(handler_name, TestDynamicRestHandler)

        self.assertItemsEqual(
            [
                ('^/{}/$'.format(handler_name), TestDynamicRestHandler),
                ('^/{}$'.format(handler_name), TestDynamicRestHandler),
                ('^/{}/toto/$'.format(handler_name), TestDynamicRestHandler),
                ('^/{}/toto$'.format(handler_name), TestDynamicRestHandler),
                ('^/{}/(?P<key>[^/]+)/$'.format(handler_name), TestDynamicRestHandler),
                ('^/{}/(?P<key>[^/]+)$'.format(handler_name), TestDynamicRestHandler)
            ],
            router.urls
        )

    def test_basic_router(self):
        router = Router()
        handler_name = 'handler'
        router.register(handler_name, TestHandler)

        self.assertItemsEqual(
            [
                ('^/{}$'.format(handler_name), TestHandler),
                ('^/{}/$'.format(handler_name), TestHandler),
                ('^/{}/(?P<key>[^/]+)$'.format(handler_name), TestHandler),
                ('^/{}/(?P<key>[^/]+)/$'.format(handler_name), TestHandler)
            ],
            router.urls
        )

    def test_model_router(self):
        router = Router()
        handler_name = 'handler'
        router.register(handler_name, TestRestHandler)

        self.assertItemsEqual(
            [
                ('^/{}/$'.format(handler_name), TestRestHandler),
                ('^/{}$'.format(handler_name), TestRestHandler),
                ('^/{}/(?P<key>[^/]+)/$'.format(handler_name), TestRestHandler),
                ('^/{}/(?P<key>[^/]+)$'.format(handler_name), TestRestHandler)
            ],
            router.urls
        )


if __name__ == '__main__':
    unittest.main()
